@@ -4,7 +4,7 @@ package POPFile::API;
 #
 # API.pm --  The API to POPFile available through XML-RPC
 #
-# Copyright (c) 2001-2009 John Graham-Cumming
+# Copyright (c) 2001-2011 John Graham-Cumming
 #
 #   This file is part of POPFile
 #
@@ -31,7 +31,7 @@ sub new
     # This will store a reference to the classifier object
 
     $self->{c} = 0;
-        
+
     bless $self, $type;
     return $self;
 }
@@ -126,6 +126,13 @@ sub handle_message
 {
     my ( $self, $session, $in, $out ) = @_;
 
+    return undef if ( !-f $in );
+
+    # Examine the session key is valid
+
+    my @buckets = $self->{c}->get_buckets( $session );
+    return undef if ( !defined( $buckets[0] ) );
+
     # Convert the two files into streams that can be passed to the
     # classifier
 
