@@ -11,7 +11,7 @@ use POPFile::Module;
 # register specific parameters with this module.  This module also handles
 # POPFile's command line parsing
 #
-# Copyright (c) 2001-2009 John Graham-Cumming
+# Copyright (c) 2001-2011 John Graham-Cumming
 #
 #   This file is part of POPFile
 #
@@ -389,7 +389,7 @@ sub parse_command_line
         push @options, ("-$1");
         if ( defined( $2 ) ) {
             push @options, ($2);
-	}
+        }
     }
 
     push @options, @ARGV;
@@ -534,7 +534,9 @@ sub load_configuration
 
     $self->{started__} = 1;
 
-    if ( open CONFIG, '<' . $self->get_user_path( 'popfile.cfg' ) ) {
+    my $config_file = $self->get_user_path( 'popfile.cfg' );
+
+    if ( open CONFIG, '<', $config_file ) {
         while ( <CONFIG> ) {
             s/(\015|\012)//g;
             if ( /(\S+) (.+)?/ ) {
@@ -561,6 +563,10 @@ sub load_configuration
         }
 
         close CONFIG;
+    } else {
+        if ( -e $config_file && !-r _ ) {
+            $self->log_( 0, "Couldn't load from the configuration file $config_file" );
+        }
     }
 
     $self->{save_needed__} = 0;
@@ -582,7 +588,14 @@ sub save_configuration
         return;
     }
 
-    if ( open CONFIG, '>' . $self->get_user_path( 'popfile.cfg.tmp' ) ) {
+    my $config_file = $self->get_user_path( 'popfile.cfg' );
+    my $config_temp = $self->get_user_path( 'popfile.cfg.tmp' );
+
+    if ( -e $config_file && !-w _ ) {
+        $self->log_( 0, "Can't write to the configuration file $config_file" );
+    }
+
+    if ( open CONFIG, '>', $config_temp ) {
         $self->{save_needed__} = 0;
 
         foreach my $key (sort keys %{$self->{configuration_parameters__}}) {
@@ -591,8 +604,9 @@ sub save_configuration
 
         close CONFIG;
 
-        rename $self->get_user_path( 'popfile.cfg.tmp' ), # PROFILE BLOCK START
-               $self->get_user_path( 'popfile.cfg' );     # PROFILE BLOCK STOP
+        rename $config_temp, $config_file;
+    } else {
+        $self->log_( 0, "Couldn't open a temporary configuration file $config_temp" );
     }
 }
 
