@@ -9,7 +9,7 @@ use POPFile::Module;
 # This module handles POPFile's history.  It manages entries in the POPFile
 # database and on disk that store messages previously classified by POPFile.
 #
-# Copyright (c) 2004-2006 John Graham-Cumming
+# Copyright (c) 2001-2011 John Graham-Cumming
 #
 #   This file is part of POPFile
 #
@@ -35,8 +35,9 @@ use locale;
 use Date::Parse;
 use Digest::MD5 qw( md5_hex );
 
-my $fields_slot = 'history.id, hdr_from, hdr_to, hdr_cc, hdr_subject,
-hdr_date, hash, inserted, buckets.name, usedtobe, history.bucketid, magnets.val, size';
+my $fields_slot =                                              # PROFILE BLOCK START
+'history.id, hdr_from, hdr_to, hdr_cc, hdr_subject, hdr_date, hash, inserted,
+ buckets.name, usedtobe, history.bucketid, magnets.val, size'; # PROFILE BLOCK STOP
 
 #----------------------------------------------------------------------------
 # new
@@ -138,6 +139,11 @@ sub stop
     # added to the queue and before service() is called
 
     $self->commit_history__();
+
+    if ( defined( $self->{db__} ) ) {
+        $self->{db__}->disconnect;
+        $self->{db__} = undef;
+    }
 }
 
 #----------------------------------------------------------------------------
@@ -271,47 +277,43 @@ sub forked
 # going to be used) or commit_slot (if the file has been written and the
 # entry should be added to the history).
 #
+# The only parameter is optional and exists for the sake of the test-
+# suite: you can pass in the time at which the message was inserted,
+# ie. the time at which the message arrived.
 #----------------------------------------------------------------------------
 sub reserve_slot
 {
-    my ( $self ) = @_;
+    my $self = shift;
+    my $inserted_time = shift || time;
 
-    my $r;
+    my $insert_sth = $self->db__()->prepare(                          # PROFILE BLOCK START
+            'insert into history ( userid, committed, inserted )
+                         values  (      ?,         ?,        ? );' ); # PROFILE BLOCK STOP
+    my $is_sqlite2 = ( $self->db__()->{Driver}->{Name} =~ /SQLite2?/ ) &&  # PROFILE BLOCK START
+                     ( $self->db__()->{sqlite_version} =~ /^2\./ );        # PROFILE BLOCK STOP
 
-    while (1) {
-        $r = int(rand( 1000000000 )+2);
+    my $slot;
+
+    while ( !defined($slot) || $slot == 0 ) {
+        my $r = int(rand( 1000000000 )+2);
 
         $self->log_( 2, "reserve_slot selected random number $r" );
 
-        # Avoid another POPFile process using the same committed id
-        $self->db__()->begin_work;
-
-        # TODO Replace the hardcoded user ID 1 with the looked up
-        # user ID from the session key
-
-        my $test = $self->db__()->selectrow_arrayref(
-                 "select id from history where committed = $r limit 1;");
-
-        if ( defined( $test ) ) {
-            $self->db__()->commit;
-            next;
-        }
-
         # Get the date/time now which will be stored in the database
         # so that we can sort on the Date: header in the message and
         # when we received it
 
-        my $now = time;
-        $self->db__()->do(
-            "insert into history ( userid, committed, inserted ) values ( 1, $r, $now );" );
-        last;
+        my $result = $insert_sth->execute( 1, $r, $inserted_time );
+        next if ( !defined( $result ) );
+
+        if ( $is_sqlite2 ) {
+            $slot = $self->db__()->func( 'last_insert_rowid' );
+        } else {
+            $slot = $self->db__()->last_insert_id( undef, undef, 'history', 'id' );
+        }
     }
 
-    my $result = $self->db__()->selectrow_arrayref(
-                 "select id from history where committed = $r limit 1;");
-    $self->db__()->commit;
-
-    my $slot = $result->[0];
+    $insert_sth->finish;
 
     $self->log_( 2, "reserve_slot returning slot id $slot" );
 
@@ -335,34 +337,34 @@ sub release_slot
     # Remove the entry from the database and delete the file
     # if present
 
-    my $delete = "delete from history where history.id = $slot;";
+    my $delete = 'delete from history where history.id = ?;';
 
-    $self->db__()->do( $delete );
+    my $h = $self->db__()->prepare( $delete );
+    $h->execute( $slot );
 
     my $file = $self->get_slot_file( $slot );
 
     unlink $file;
 
-    # It's not possible that the directory for the slot file is empty
-    # and we want to delete it so that things get cleaned up automatically
+    # It's now possible that the directory for the slot file is empty
+    # and we want to delete it so that things get cleaned up
+    # automatically
 
-    $file =~ s/popfile[a-f0-9]{2}\.msg$//i;
+    my $directory = $file;
+    $directory =~ s/popfile[a-f0-9]{2}\.msg$//i;
 
     my $depth = 3;
 
     while ( $depth > 0 ) {
-        my @files = glob( $file . '*' );
-
-        if ( $#files == -1 ) {
-            if ( !( rmdir( $file ) ) ) {
-                last;
-            }
-            $file =~ s![a-f0-9]{2}/$!!i;
-        } else {
+        if ( rmdir( $directory ) ) {
+            $directory =~ s![a-f0-9]{2}/$!!i;
+            $depth--;
+        }
+        else {
+            # We either aren't allowed to delete the
+            # directory or it wasn't empty
             last;
         }
-
-        $depth--;
     }
 }
 
@@ -412,8 +414,8 @@ sub change_slot_classification
     # then retrieve the current classification for this slot
     # and update the database
 
-    my $bucketid = $self->{classifier__}->get_bucket_id(
-                           $session, $class );
+    my $bucketid = $self->{classifier__}->get_bucket_id(  # PROFILE BLOCK START
+                           $session, $class );            # PROFILE BLOCK STOP
 
     my $oldbucketid = 0;
     if ( !$undo ) {
@@ -421,10 +423,12 @@ sub change_slot_classification
         $oldbucketid = $fields[10];
     }
 
-    $self->db__()->do( "update history set bucketid = $bucketid,
-                                           usedtobe = $oldbucketid
-                                       where id = $slot;" );
-    $self->force_requery__();
+    my $h = $self->db__()->prepare(  # PROFILE BLOCK START
+        'update history set bucketid = ?,
+                            usedtobe = ?
+                where id = ?;' );    # PROFILE BLOCK STOP
+    $h->execute( $bucketid, $oldbucketid, $slot );
+    $self->force_requery();
 }
 
 #----------------------------------------------------------------------------
@@ -444,10 +448,12 @@ sub revert_slot_classification
     my @fields = $self->get_slot_fields( $slot );
     my $oldbucketid = $fields[9];
 
-    $self->db__()->do( "update history set bucketid = $oldbucketid,
-                                           usedtobe = 0
-                                       where id = $slot;" );
-    $self->force_requery__();
+    my $h = $self->db__()->prepare(  # PROFILE BLOCK START
+        'update history set bucketid = ?,
+                            usedtobe = ?
+                where id = ?;' );    # PROFILE BLOCK STOP
+    $h->execute( $oldbucketid, 0, $slot );
+    $self->force_requery();
 }
 
 #---------------------------------------------------------------------------
@@ -464,11 +470,17 @@ sub get_slot_fields
 {
     my ( $self, $slot ) = @_;
 
-    return $self->db__()->selectrow_array(
+    return undef if ( !defined( $slot ) || $slot !~ /^\d+$/ );
+
+    my $h = $self->db__()->prepare(           # PROFILE BLOCK START
         "select $fields_slot from history, buckets, magnets
-             where history.id = $slot and
-                   buckets.id = history.bucketid and
-                   magnets.id = magnetid;" );
+             where history.id        = ? and
+                   buckets.id        = history.bucketid and
+                   magnets.id        = magnetid and
+                   history.committed = 1;" ); # PROFILE BLOCK STOP
+    $h->execute( $slot );
+    my @result = $h->fetchrow_array;
+    return @result;
 }
 
 #---------------------------------------------------------------------------
@@ -484,8 +496,14 @@ sub is_valid_slot
 {
     my ( $self, $slot ) = @_;
 
-    my @row = $self->db__()->selectrow_array(
-        "select id from history where history.id = $slot;" );
+    return undef if ( !defined( $slot ) || $slot !~ /^\d+$/ );
+
+    my $h = $self->db__()->prepare(           # PROFILE BLOCK START
+        'select id from history
+             where history.id        = ? and
+                   history.committed = 1;' ); # PROFILE BLOCK STOP
+    $h->execute( $slot );
+    my @row = $h->fetchrow_array;
 
     return ( ( @row ) && ( $row[0] == $slot ) );
 }
@@ -506,6 +524,23 @@ sub commit_history__
         return;
     }
 
+    my $update_history = $self->db__()->prepare(         # PROFILE BLOCK START
+                'update history set hdr_from    = ?,
+                                    hdr_to      = ?,
+                                    hdr_date    = ?,
+                                    hdr_cc      = ?,
+                                    hdr_subject = ?,
+                                    sort_from   = ?,
+                                    sort_to     = ?,
+                                    sort_cc     = ?,
+                                    committed   = ?,
+                                    bucketid    = ?,
+                                    usedtobe    = ?,
+                                    magnetid    = ?,
+                                    hash        = ?,
+                                    size        = ?
+                                    where id    = ?;' ); # PROFILE BLOCK STOP
+
     $self->db__()->begin_work;
     foreach my $entry (@{$self->{commit_list__}}) {
         my ( $session, $slot, $bucket, $magnet ) = @{$entry};
@@ -550,7 +585,6 @@ sub commit_history__
                                             ${$header{'date'}}[0],
                                             ${$header{'subject'}}[0],
                                             ${$header{'received'}}[0] );
-        $hash = $self->db__()->quote( $hash );
 
         # For sorting purposes the From, To and CC headers have special
         # cleaned up versions of themselves in the database.  The idea
@@ -564,15 +598,13 @@ sub commit_history__
         my %sort_headers;
 
         foreach my $h (@sortable) {
-            $sort_headers{$h} =
+            $sort_headers{$h} =           # PROFILE BLOCK START
                  $self->{classifier__}->{parser__}->decode_string(
-                     ${$header{$h}}[0] );
+                     ${$header{$h}}[0] ); # PROFILE BLOCK STOP
             $sort_headers{$h} = lc($sort_headers{$h} || '');
             $sort_headers{$h} =~ s/[\"<>]//g;
             $sort_headers{$h} =~ s/^[ \t]+//g;
             $sort_headers{$h} =~ s/\0//g;
-            $sort_headers{$h} = $self->db__()->quote(
-                $sort_headers{$h} );
         }
 
         # Make sure that the headers we are going to insert into
@@ -581,11 +613,10 @@ sub commit_history__
         my @required = ( 'from', 'to', 'cc', 'subject' );
 
         foreach my $h (@required) {
-
-            ${$header{$h}}[0] =
+            ${$header{$h}}[0] =           # PROFILE BLOCK START
                  $self->{classifier__}->{parser__}->decode_string(
-                     ${$header{$h}}[0] );
-            
+                     ${$header{$h}}[0] ); # PROFILE BLOCK STOP
+
             if ( !defined ${$header{$h}}[0] || ${$header{$h}}[0] =~ /^\s*$/ ) {
                 if ( $h ne 'cc' ) {
                     ${$header{$h}}[0] = "<$h header missing>";
@@ -593,9 +624,8 @@ sub commit_history__
                     ${$header{$h}}[0] = '';
                 }
             }
-            
+
             ${$header{$h}}[0] =~ s/\0//g;
-            ${$header{$h}}[0] = $self->db__()->quote( ${$header{$h}}[0] );
         }
 
         # If we do not have a date header then set the date to
@@ -613,8 +643,8 @@ sub commit_history__
         # classified into (and the same for the magnet if it is
         # defined)
 
-        my $bucketid = $self->{classifier__}->get_bucket_id(
-                           $session, $bucket );
+        my $bucketid = $self->{classifier__}->get_bucket_id(  # PROFILE BLOCK START
+                           $session, $bucket );               # PROFILE BLOCK STOP
 
         my $msg_size = -s $file;
 
@@ -624,31 +654,33 @@ sub commit_history__
         # history and log the failure
 
         if ( defined( $bucketid ) ) {
-            my $result = $self->db__()->do(
-                "update history set hdr_from    = ${$header{from}}[0],
-                                    hdr_to      = ${$header{to}}[0],
-                                    hdr_date    = ${$header{date}}[0],
-                                    hdr_cc      = ${$header{cc}}[0],
-                                    hdr_subject = ${$header{subject}}[0],
-                                    sort_from   = $sort_headers{from},
-                                    sort_to     = $sort_headers{to},
-                                    sort_cc     = $sort_headers{cc},
-                                    committed   = 1,
-                                    bucketid    = $bucketid,
-                                    usedtobe    = 0,
-                                    magnetid    = $magnet,
-                                    hash        = $hash,
-                                    size        = $msg_size
-                                    where id = $slot;" );
+            my $result = $update_history->execute(  # PROFILE BLOCK START
+                    ${$header{from}}[0],    # hdr_from
+                    ${$header{to}}[0],      # hdr_to
+                    ${$header{date}}[0],    # hdr_date
+                    ${$header{cc}}[0],      # hdr_cc
+                    ${$header{subject}}[0], # hdr_subject
+                    $sort_headers{from},    # sort_from
+                    $sort_headers{to},      # sort_to
+                    $sort_headers{cc},      # sort_cc
+                    1,                      # committed
+                    $bucketid,              # bucketid
+                    0,                      # usedtobe
+                    $magnet,                # magnetid
+                    $hash,                  # hash
+                    $msg_size,              # size
+                    $slot                   # id
+            );                                      # PROFILE BLOCK STOP
         } else {
             $self->log_( 0, "Couldn't find bucket ID for bucket $bucket when committing $slot" );
             $self->release_slot( $slot );
         }
     }
     $self->db__()->commit;
+    $update_history->finish;
 
     $self->{commit_list__} = ();
-    $self->force_requery__();
+    $self->force_requery();
 }
 
 # ---------------------------------------------------------------------------
@@ -674,15 +706,15 @@ sub delete_slot
 
         $self->make_directory__( $path );
 
-        my @b = $self->db__()->selectrow_array(
+        my $b = $self->db__()->selectrow_arrayref(
             "select buckets.name from history, buckets
                  where history.bucketid = buckets.id and
                        history.id = $slot;" );
 
-        my $bucket = $b[0];
+        my $bucket = $b->[0];
 
-        if ( ( $bucket ne 'unclassified' ) &&
-             ( $bucket ne 'unknown class' ) ) {
+        if ( ( $bucket ne 'unclassified' ) &&   # PROFILE BLOCK START
+             ( $bucket ne 'unknown class' ) ) { # PROFILE BLOCK STOP
             $path .= "\/" . $bucket;
             $self->make_directory__( $path );
 
@@ -690,8 +722,8 @@ sub delete_slot
 
                 # Archive to a random sub-directory of the bucket archive
 
-                my $subdirectory = int( rand(
-                    $self->config_( 'archive_classes' ) ) );
+                my $subdirectory = int( rand(                # PROFILE BLOCK START
+                    $self->config_( 'archive_classes' ) ) ); # PROFILE BLOCK STOP
                 $path .= "\/" . $subdirectory;
                 $self->make_directory__( $path );
             }
@@ -711,7 +743,7 @@ sub delete_slot
     # may have been affected
 
     $self->release_slot( $slot );
-    $self->force_requery__();
+    $self->force_requery();
 }
 
 #----------------------------------------------------------------------------
@@ -727,7 +759,7 @@ sub start_deleting
 {
     my ( $self ) = @_;
 
-    $self->{classifier__}->tweak_sqlite( 1, 1, $self->db__() );
+#    $self->{classifier__}->tweak_sqlite( 1, 1, $self->db__() );
     $self->db__()->begin_work;
 }
 
@@ -744,7 +776,7 @@ sub stop_deleting
     my ( $self ) = @_;
 
     $self->db__()->commit;
-    $self->{classifier__}->tweak_sqlite( 1, 0, $self->db__() );
+#    $self->{classifier__}->tweak_sqlite( 1, 0, $self->db__() );
 }
 
 #----------------------------------------------------------------------------
@@ -772,9 +804,9 @@ sub get_slot_file
     # Hence each directory can have up to 256 entries
 
     my $hex_slot = sprintf( '%8.8x', $slot );
-    my $path = $self->get_user_path_(
+    my $path = $self->get_user_path_(                        # PROFILE BLOCK START
                    $self->global_config_( 'msgdir' ) .
-                       substr( $hex_slot, 0, 2 ) . '/', 0 );
+                       substr( $hex_slot, 0, 2 ) . '/', 0 ); # PROFILE BLOCK STOP
 
     $self->make_directory__( $path );
     $path .= substr( $hex_slot, 2, 2 ) . '/';
@@ -782,8 +814,8 @@ sub get_slot_file
     $path .= substr( $hex_slot, 4, 2 ) . '/';
     $self->make_directory__( $path );
 
-    my $file = 'popfile' .
-               substr( $hex_slot, 6, 2 ) . '.msg';
+    my $file = 'popfile' .                         # PROFILE BLOCK START
+               substr( $hex_slot, 6, 2 ) . '.msg'; # PROFILE BLOCK STOP
 
     return $path . $file;
 }
@@ -834,9 +866,10 @@ sub get_slot_from_hash
 {
     my ( $self, $hash ) = @_;
 
-    $hash = $self->db__()->quote( $hash );
-    my $result = $self->db__()->selectrow_arrayref(
-        "select id from history where hash = $hash limit 1;" );
+    my $h = $self->db__()->prepare(                         # PROFILE BLOCK START
+        'select id from history where hash = ? limit 1;' ); # PROFILE BLOCK STOP
+    $h->execute( $hash );
+    my $result = $h->fetchrow_arrayref;
 
     return defined( $result )?$result->[0]:'';
 }
@@ -908,9 +941,10 @@ sub stop_query
     my $q = $self->{queries__}{$id}{query};
 
     if ( ( defined $q ) && ( $q != 0 ) ) {
-        if ( $#{$self->{queries__}{$id}{cache}} !=
-             $self->{queries__}{$id}{count} ) {
-           $q->finish;
+        if ( $#{$self->{queries__}{$id}{cache}} !=  # PROFILE BLOCK START
+             $self->{queries__}{$id}{count} ) {     # PROFILE BLOCK STOP
+            $q->finish;
+            undef $self->{queries__}{$id}{query};
         }
     }
 
@@ -935,12 +969,15 @@ sub set_query
 {
     my ( $self, $id, $filter, $search, $sort, $not ) = @_;
 
+    $search =~ s/\0//g;
+    $sort = '' if ( $sort !~ /^(\-)?(inserted|from|to|cc|subject|bucket|date|size)$/ );
+
     # If this query has already been done and is in the cache
     # then do no work here
 
-    if ( defined( $self->{queries__}{$id}{fields} ) &&
+    if ( defined( $self->{queries__}{$id}{fields} ) &&  # PROFILE BLOCK START
          ( $self->{queries__}{$id}{fields} eq
-             "$filter:$search:$sort:$not" ) ) {
+             "$filter:$search:$sort:$not" ) ) {         # PROFILE BLOCK STOP
         return;
     }
 
@@ -951,8 +988,9 @@ sub set_query
     # so that we know the size of the resulting data without having
     # to retrieve it all
 
-    $self->{queries__}{$id}{base} = 'select XXX from
-        history, buckets, magnets where history.userid = 1 and committed = 1';
+    $self->{queries__}{$id}{base} =                          # PROFILE BLOCK START
+        'select XXX from history, buckets, magnets
+                where history.userid = 1 and committed = 1'; # PROFILE BLOCK STOP
 
     $self->{queries__}{$id}{base} .= ' and history.bucketid = buckets.id';
     $self->{queries__}{$id}{base} .= ' and magnets.id = magnetid';
@@ -960,9 +998,9 @@ sub set_query
     # If there's a search portion then add the appropriate clause
     # to find the from/subject header
 
-    my $not_word  = $not?'not':'';
-    my $not_equal = $not?'!=':'=';
-    my $equal     = $not?'=':'!=';
+    my $not_word  = $not ? 'not' : '';
+    my $not_equal = $not ? '!='  : '=';
+    my $equal     = $not ? '='   : '!=';
 
     if ( $search ne '' ) {
         $search = $self->db__()->quote( '%' . $search . '%' );
@@ -974,16 +1012,17 @@ sub set_query
 
     if ( $filter ne '' ) {
         if ( $filter eq '__filter__magnet' ) {
-            $self->{queries__}{$id}{base} .=
-                " and history.magnetid $equal 0";
+            $self->{queries__}{$id}{base} .=      # PROFILE BLOCK START
+                " and history.magnetid $equal 0"; # PROFILE BLOCK STOP
         } else {
-            my $session = $self->{classifier__}->get_session_key(
-                              'admin', '' );
-            my $bucketid = $self->{classifier__}->get_bucket_id(
-                               $session, $filter );
-            $self->{classifier__}->release_session_key( $session );
-            $self->{queries__}{$id}{base} .=
-                " and history.bucketid $not_equal $bucketid";
+            if ( $filter eq '__filter__reclassified' ) {
+                $self->{queries__}{$id}{base} .=      # PROFILE BLOCK START
+                    " and history.usedtobe $equal 0"; # PROFILE BLOCK STOP
+            } else {
+                my $bucket = $self->db__()->quote( $filter );
+                $self->{queries__}{$id}{base} .=
+                        " and buckets.name $not_equal $bucket";
+            }
         }
     }
 
@@ -1012,13 +1051,14 @@ sub set_query
     $self->log_( 2, "Base query is $count" );
     $count =~ s/XXX/COUNT(*)/;
 
-    $self->{queries__}{$id}{count} =
-        $self->db__()->selectrow_arrayref( $count )->[0];
+    my $h = $self->db__()->prepare( $count );
+    $h->execute;
+    $self->{queries__}{$id}{count} = $h->fetchrow_arrayref->[0];
+    $h->finish;
 
     my $select = $self->{queries__}{$id}{base};
     $select =~ s/XXX/$fields_slot/;
     $self->{queries__}{$id}{query} = $self->db__()->prepare( $select );
-    $self->{queries__}{$id}{query}->execute;
     $self->{queries__}{$id}{cache} = ();
 }
 
@@ -1041,10 +1081,12 @@ sub delete_query
     $delete =~ s/XXX/history.id/;
     my $d = $self->db__()->prepare( $delete );
     $d->execute;
+    my $history_id;
     my @row;
     my @ids;
-    while ( @row = $d->fetchrow_array ) {
-        push ( @ids, $row[0] );
+    $d->bind_columns( \$history_id );
+    while ( $d->fetchrow_arrayref ) {
+        push ( @ids, $history_id );
     }
     foreach my $id (@ids) {
         $self->delete_slot( $id, 1 );
@@ -1102,9 +1144,11 @@ sub get_query_rows
     if ( ( $size < ( $start + $count - 1 ) ) ) {
         my $rows = $start + $count - $size;
         $self->log_( 2, "Getting $rows rows from database" );
-        push ( @{$self->{queries__}{$id}{cache}},
-            @{$self->{queries__}{$id}{query}->fetchall_arrayref(
-                undef, $start + $count - $size )} );
+        $self->{queries__}{$id}{query}->execute;
+        $self->{queries__}{$id}{cache} =      # PROFILE BLOCK START
+            $self->{queries__}{$id}{query}->fetchall_arrayref(
+                undef, $start + $count - 1 ); # PROFILE BLOCK STOP
+        $self->{queries__}{$id}{query}->finish;
     }
 
     my ( $from, $to ) = ( $start-1, $start+$count-2 );
@@ -1172,8 +1216,8 @@ sub upgrade_history_files__
     # See if there are any .MSG files in the msgdir, and if there are
     # upgrade them by placing them in the database
 
-    my @msgs = sort compare_mf__ glob $self->get_user_path_(
-        $self->global_config_( 'msgdir' ) . 'popfile*.msg', 0 );
+    my @msgs = sort compare_mf__ glob $self->get_user_path_(     # PROFILE BLOCK START
+        $self->global_config_( 'msgdir' ) . 'popfile*.msg', 0 ); # PROFILE BLOCK STOP
 
     if ( $#msgs != -1 ) {
         my $session = $self->{classifier__}->get_session_key( 'admin', '' );
@@ -1193,8 +1237,8 @@ sub upgrade_history_files__
             # in upgraded history.  Also the $magnet is ignored so
             # upgraded history will have no magnet information.
 
-            my ( $reclassified, $bucket, $usedtobe, $magnet ) =
-                $self->history_read_class__( $msg );
+            my ( $reclassified, $bucket, $usedtobe, $magnet ) =  # PROFILE BLOCK START
+                $self->history_read_class__( $msg );             # PROFILE BLOCK STOP
 
             if ( $bucket ne 'unknown_class' ) {
                 my ( $slot, $file ) = $self->reserve_slot();
@@ -1210,8 +1254,8 @@ sub upgrade_history_files__
         $self->commit_history__();
         $self->{classifier__}->release_session_key( $session );
 
-        unlink $self->get_user_path_(
-            $self->global_config_( 'msgdir' ) . 'history_cache', 0 );
+        unlink $self->get_user_path_(                                 # PROFILE BLOCK START
+            $self->global_config_( 'msgdir' ) . 'history_cache', 0 ); # PROFILE BLOCK STOP
     }
 }
 
@@ -1244,8 +1288,8 @@ sub history_read_class__
 
     if ( open CLASS, "<$filename" ) {
         $bucket = <CLASS>;
-        if ( defined( $bucket ) &&
-           ( $bucket =~ /([^ ]+) MAGNET ([^\r\n]+)/ ) ) {
+        if ( defined( $bucket ) &&                        # PROFILE BLOCK START
+           ( $bucket =~ /([^ ]+) MAGNET ([^\r\n]+)/ ) ) { # PROFILE BLOCK STOP
             $bucket = $1;
             $magnet = $2;
         }
@@ -1283,14 +1327,17 @@ sub cleanup_history
 
     my $seconds_per_day = 24 * 60 * 60;
     my $old = time - $self->config_( 'history_days' ) * $seconds_per_day;
-    my $d = $self->db__()->prepare( "select id from history
-                                         where inserted < $old;" );
-    $d->execute;
-    my @row;
     my @ids;
-    while ( @row = $d->fetchrow_array ) {
-        push ( @ids, $row[0] );
+    my $d = $self->db__()->prepare(     # PROFILE BLOCK START
+        'select id from history
+                where inserted < ?;' ); # PROFILE BLOCK STOP
+    $d->execute( $old );
+    my $id;
+    $d->bind_columns( \$id );
+    while ( $d->fetchrow_arrayref ) {
+        push ( @ids, $id );
     }
+    $d->finish;
     foreach my $id (@ids) {
         $self->delete_slot( $id, 1 );
     }
@@ -1328,13 +1375,13 @@ sub copy_file__
 
 # ---------------------------------------------------------------------------
 #
-# force_requery__
+# force_requery
 #
 # Called when the database has changed to invalidate any queries that are
 # open so that cached data is not returned and the database is requeried
 #
 # ---------------------------------------------------------------------------
-sub force_requery__
+sub force_requery
 {
     my ( $self ) = @_;
     # Force requery since the messages have changed
