@@ -11,7 +11,7 @@ package POPFile::Loader;
 # Subroutines not so marked are suitable for use by POPFile-based
 # utilities to assist in loading and executing modules
 #
-# Copyright (c) 2001-2006 John Graham-Cumming
+# Copyright (c) 2001-2011 John Graham-Cumming
 #
 #   This file is part of POPFile
 #
@@ -32,6 +32,8 @@ package POPFile::Loader;
 #
 #----------------------------------------------------------------------------
 
+use Getopt::Long qw(:config pass_through);
+
 #----------------------------------------------------------------------------
 # new
 #
@@ -61,6 +63,12 @@ sub new
 
     $self->{debug__} = 1;
 
+    # If this is set to 1 then POPFile will shutdown straight after it
+    # has started up.  This is used by the installer and set by the
+    # --shutdown command-line option
+
+    $self->{shutdown__} = 0;
+
     # This stuff lets us do some things in a way that tolerates some
     # window-isms
 
@@ -77,6 +85,9 @@ sub new
     $self->{pipeready__}    = '';
     $self->{forker__}       = '';
     $self->{reaper__}       = '';
+    $self->{childexit__}    = '';
+    $self->{warning__}      = '';
+    $self->{die__}          = '';
 
     # POPFile's version number as individual numbers and as
     # string
@@ -115,10 +126,13 @@ sub CORE_loader_init
     # to $self, without exposing $self to the unwashed. No reference to
     # POPFile::Loader is needed by the caller
 
-    $self->{aborting__} = sub { $self->CORE_aborting(@_) };
+    $self->{aborting__}  = sub { $self->CORE_aborting(@_) };
     $self->{pipeready__} = sub { $self->pipeready(@_) };
-    $self->{forker__} = sub { $self->CORE_forker(@_) };
-    $self->{reaper__} = sub { $self->CORE_reaper(@_) };
+    $self->{forker__}    = sub { $self->CORE_forker(@_) };
+    $self->{reaper__}    = sub { $self->CORE_reaper(@_) };
+    $self->{childexit__} = sub { $self->CORE_childexit(@_) };
+    $self->{warning__}   = sub { $self->CORE_warning(@_) };
+    $self->{die__}       = sub { $self->CORE_die(@_) };
 
     # See if there's a file named popfile_version that contains the
     # POPFile version number
@@ -134,6 +148,10 @@ sub CORE_loader_init
         $self->CORE_version( $major, $minor, $rev );
     }
 
+    # Parse the command-line options (only --shutdown is supported at present) 
+
+    GetOptions( "shutdown" => \$self->{shutdown__} );
+
     print "\nPOPFile Engine loading\n" if $self->{debug__};
 }
 
@@ -218,6 +236,30 @@ sub CORE_reaper
     $SIG{CHLD} = $self->{reaper__};
 }
 
+#----------------------------------------------------------------------------
+#
+# CORE_childexit
+#
+# Called by a module that is in a child process and wants to exit.  This
+# warns all the other modules in the same process by calling their childexit
+# function and then does the exit.
+#
+# $code         The process exit code
+#
+#----------------------------------------------------------------------------
+sub CORE_childexit
+{
+    my ( $self, $code ) = @_;
+
+    foreach my $type (sort keys %{$self->{components__}}) {
+        foreach my $name (sort keys %{$self->{components__}{$type}}) {
+            $self->{components__}{$type}{$name}->childexit();
+        }
+    }
+
+    exit( $code );
+}
+
 #----------------------------------------------------------------------------
 #
 # CORE_forker
@@ -236,9 +278,11 @@ sub CORE_forker
 {
     my ( $self ) = @_;
 
+    my @types = sort keys %{$self->{components__}};
+
     # Tell all the modules that a fork is about to happen
 
-    foreach my $type (sort keys %{$self->{components__}}) {
+    foreach my $type ( @types ) {
         foreach my $name (sort keys %{$self->{components__}{$type}}) {
             $self->{components__}{$type}{$name}->prefork();
         }
@@ -266,7 +310,7 @@ sub CORE_forker
     # caller knows that we are in the child
 
     if ( $pid == 0 ) {
-          foreach my $type (sort keys %{$self->{components__}}) {
+          foreach my $type ( @types ) {
                foreach my $name (sort keys %{$self->{components__}{$type}}) {
                  $self->{components__}{$type}{$name}->forked( $writer );
               }
@@ -288,7 +332,7 @@ sub CORE_forker
     # writer pipe file handle and return our PID (non-zero) indicating
     # that this is the parent process
 
-    foreach my $type (sort keys %{$self->{components__}}) {
+    foreach my $type ( @types ) {
         foreach my $name (sort keys %{$self->{components__}{$type}}) {
             $self->{components__}{$type}{$name}->postfork( $pid, $reader );
         }
@@ -298,6 +342,54 @@ sub CORE_forker
     return ($pid, $reader);
 }
 
+#----------------------------------------------------------------------------
+#
+# CORE_warning
+#
+# Called when a warning occurs.
+# Output the warning to the log file and print it to STDERR.
+#
+#----------------------------------------------------------------------------
+sub CORE_warning
+{
+    my ( $self, @message ) = @_;
+
+    if ( $self->module_config( 'GLOBAL', 'debug' ) > 0 ) {
+        $self->{components__}{core}{logger}->debug( 0, "Perl warning: @message" );
+        warn @message;
+    }
+}
+
+#----------------------------------------------------------------------------
+#
+# CORE_die
+#
+# Called when a fatal error occurs.
+# Output the error message to the log file and exit.
+#
+#----------------------------------------------------------------------------
+sub CORE_die
+{
+    my ( $self, @message ) = @_;
+
+    # Do nothing when dies in eval
+
+    return if $^S;
+
+    # Print error message
+
+    print STDERR @message;
+
+    if ( $self->module_config( 'GLOBAL', 'debug' ) > 0 ) {
+        $self->{components__}{core}{logger}->debug( 0, "Perl fatal error : @message" );
+    }
+
+    # Try to stop safely
+
+    $self->CORE_stop( );
+    exit 1;
+}
+
 #----------------------------------------------------------------------------
 #
 # CORE_load_directory_modules
@@ -438,6 +530,14 @@ sub CORE_signals
 
     $SIG{PIPE}  = 'IGNORE';
 
+    # Better handling of the Perl warnings
+
+    $SIG{__WARN__} = $self->{warning__};
+
+    # Try to capture the Perl errors
+
+    $SIG{__DIE__} = $self->{die__};
+
     return $SIG;
 }
 
@@ -485,7 +585,7 @@ sub CORE_load
 
     # Create the main objects that form the core of POPFile.  Consists
     # of the configuration modules, the classifier, the UI (currently
-    # HTML based), and the POP3 proxy.
+    # HTML based), and the proxies.
 
     print "\n    Loading... " if $self->{debug__};
 
@@ -583,17 +683,19 @@ sub CORE_initialize
             print " $name" if $self->{debug__};
             flush STDOUT;
 
-            my $code = $self->{components__}{$type}{$name}->initialize();
+            my $mod = $self->{components__}{$type}{$name};
+
+            my $code = $mod->initialize();
 
             if ( $code == 0 ) {
                 die "Failed to start while initializing the $name module";
             }
 
             if ( $code == 1 ) {
-                 $self->{components__}{$type}{$name}->alive(     1 );
-
-                 $self->{components__}{$type}{$name}->forker(    $self->{forker__} );
-                 $self->{components__}{$type}{$name}->pipeready( $self->{pipeready__} );
+                 $mod->alive(     1 );
+                 $mod->forker(    $self->{forker__} );
+                 $mod->setchildexit( $self->{childexit__} );
+                 $mod->pipeready( $self->{pipeready__} );
 	    }
         }
         print '} ' if $self->{debug__};
@@ -708,6 +810,13 @@ sub CORE_service
         }
 
         last if $nowait;
+        
+        # If we are asked to shutdown then we allow a single run
+        # through the service routines and then exit
+
+        if ( $self->{shutdown__} == 1 ) {
+            $self->{alive__} = 0;
+        }
     }
 
     return $self->{alive__};
@@ -746,6 +855,7 @@ sub CORE_stop
         foreach my $name (sort keys %{$self->{components__}{$type}}) {
             print " $name" if $self->{debug__};
             flush STDOUT;
+
             next if ( $name eq 'mq' );
             next if ( $name eq 'history' );
             $self->{components__}{$type}{$name}->alive(0);
@@ -810,6 +920,7 @@ sub get_module
         $name = lc($2);
 
         $type =~ s/^POPFile$/core/i;
+        $type =~ s/^UI$/interface/i;
     }
 
     return $self->{components__}{$type}{$name};
