@@ -2,7 +2,7 @@
 #
 # This is POPFile's top level Module object.
 #
-# Copyright (c) 2001-2006 John Graham-Cumming
+# Copyright (c) 2001-2011 John Graham-Cumming
 #
 #   This file is part of POPFile
 #
@@ -55,7 +55,10 @@ use IO::Select;
 # called within the child process and should be used to clean up
 #
 # postfork() - called in the parent process to tell it that the fork
-# has occurred.  This is like forked but in the parent
+# has occurred.  This is like forked but in the parent.
+#
+# childexit() - called in a child process when the child is about
+# to exit.
 #
 # reaper() - called when a process has terminated to give a module a
 # chance to do whatever clean up is needed
@@ -208,7 +211,7 @@ sub start
 # stop
 #
 # Called when POPFile is closing down, this is the last method that
-# will get called before the object is destroyed.  There is not return
+# will get called before the object is destroyed.  There is no return
 # value from stop().
 #
 # ----------------------------------------------------------------------------
@@ -305,6 +308,20 @@ sub postfork
     my ( $self, $pid, $reader ) = @_;
 }
 
+# ----------------------------------------------------------------------------
+#
+# childexit
+#
+# Called in a child process when the child is about to exit
+#
+# There is no return value from this method
+#
+# ----------------------------------------------------------------------------
+sub childexit
+{
+    my ( $self ) = @_;
+}
+
 # ----------------------------------------------------------------------------
 #
 # deliver
@@ -337,8 +354,7 @@ sub log_
     my ( $self, $level, $message ) = @_;
 
     my ( $package, $file, $line ) = caller;
-    $self->{logger__}->debug( $level, $self->{name__} . ": $line: " .
-        $message );
+    $self->{logger__}->debug( $level, "$self->{name__}: $line: $message" );
 }
 
 # ----------------------------------------------------------------------------
@@ -523,25 +539,7 @@ sub flush_slurp_data__
 
         if ( $slurp_data__{"$handle"}{data} eq '' ) {
 
-            # This unpleasant boolean is to handle the case where we
-            # are slurping a non-socket stream under Win32
-
-            my $can_read;
-
-            $can_read = ( ( $handle !~ /socket/i ) && ( $^O eq 'MSWin32' ) );
-
-            if ( !$can_read ) {
-                if ( $handle =~ /ssl/i ) {
-                    # If using SSL, check internal buffer of OpenSSL first.
-                    $can_read = ( $handle->pending() > 0 );
-                }
-                if ( !$can_read ) {
-                    $can_read = defined( $slurp_data__{"$handle"}{select}->can_read( $self->global_config_( 'timeout' ) ) );
-                }
-            }
- 
-            if ( $can_read ) {
-
+            if ( $self->can_read__( $handle )) {
                 my $c;
                 my $retcode = sysread( $handle, $c, 1 );
                 if ( $retcode == 1 ) {
@@ -549,10 +547,10 @@ sub flush_slurp_data__
                         $cr .= $c;
                     } else {
                         $slurp_data__{"$handle"}{data} = $c;
-		    }
-		}
-	    }
-	}
+                    }
+                }
+            }
+        }
 
         return $cr;
     }
@@ -596,7 +594,7 @@ sub slurp_buffer_
 
     while ( $self->slurp_data_size__( $handle ) < $length ) {
         my $c;
-        if ( sysread( $handle, $c, $length ) > 0 ) {
+        if ( $self->can_read__( $handle, 0.01 ) && ( sysread( $handle, $c, $length ) > 0 ) ) {
             $slurp_data__{"$handle"}{data} .= $c;
         } else {
             last;
@@ -610,8 +608,8 @@ sub slurp_buffer_
         $slurp_data__{"$handle"}{data} = '';
     } else {
         $result = substr( $slurp_data__{"$handle"}{data}, 0, $length );
-        $slurp_data__{"$handle"}{data} =
-            substr( $slurp_data__{"$handle"}{data}, $length );
+        $slurp_data__{"$handle"}{data} =                       # PROFILE BLOCK START
+            substr( $slurp_data__{"$handle"}{data}, $length ); # PROFILE BLOCK STOP
     }
 
     return ($result ne '')?$result:undef;
@@ -637,7 +635,9 @@ sub slurp_buffer_
 # ----------------------------------------------------------------------------
 sub slurp_
 {
-    my ( $self, $handle ) = @_;
+    my ( $self, $handle, $timeout ) = @_;
+
+    $timeout = $self->global_config_( 'timeout' ) if ( !defined( $timeout ) );
 
     if ( !defined( $slurp_data__{"$handle"}{data} ) ) {
         $slurp_data__{"$handle"}{select} = new IO::Select( $handle );
@@ -652,16 +652,25 @@ sub slurp_
 
     my $c;
 
-    while ( sysread( $handle, $c, 160 ) > 0 ) {
-        $slurp_data__{"$handle"}{data} .= $c;
+    if ( $self->can_read__( $handle, $timeout ) ) {
+        while ( sysread( $handle, $c, 160 ) > 0 ) {
+            $slurp_data__{"$handle"}{data} .= $c;
 
-        $self->log_( 2, "Read slurp data $c" );
+            $self->log_( 2, "Read slurp data $c" );
 
-        $result = $self->flush_slurp_data__( $handle );
+            $result = $self->flush_slurp_data__( $handle );
 
-        if ( $result ne '' ) {
-            return $result;
+            if ( $result ne '' ) {
+                return $result;
+            }
         }
+    } else {
+
+        # Server has not respond. Close the connection and return
+
+        $self->done_slurp_( $handle );
+        close $handle;
+        return undef;
     }
 
     # If we get here with something in line then the file ends without any
@@ -766,6 +775,46 @@ sub flush_extra_
    return $full_buf;
 }
 
+# ----------------------------------------------------------------------------
+#
+# can_read__ - Check whether we can read from the specified handle
+#
+# Returns true if we can read from the handle
+#
+# $handle      A connection handle
+# $timeout     A timeout period (in seconds)
+#
+# ----------------------------------------------------------------------------
+sub can_read__
+{
+    my ( $self, $handle, $timeout ) = @_;
+    $timeout = $self->global_config_( 'timeout' ) if ( !defined($timeout) );
+
+    # This unpleasant boolean is to handle the case where we
+    # are slurping a non-socket stream under Win32
+
+    my $can_read = ( ( $handle !~ /socket/i ) && ( $^O eq 'MSWin32' ) );
+
+    if ( !$can_read ) {
+        if ( $handle =~ /ssl/i ) {
+
+            # If using SSL, check internal buffer of OpenSSL first.
+
+            $can_read = ( $handle->pending() > 0 );
+        }
+        if ( !$can_read ) {
+            if ( defined( $slurp_data__{"$handle"}{select} ) ) {
+                $can_read = defined( $slurp_data__{"$handle"}{select}->can_read( $timeout ) );
+            } else {
+                my $selector    = new IO::Select( $handle );
+                $can_read = defined( $selector->can_read( $timeout ) );
+            }
+        }
+    }
+
+    return $can_read;
+}
+
 # GETTER/SETTER methods.  Note that I do not expect documentation of
 # these unless they are non-trivial since the documentation would be a
 # waste of space
@@ -831,6 +880,17 @@ sub logger
     return $self->{logger__};
 }
 
+sub setchildexit
+{
+    my ( $self, $value ) = @_;
+
+    if ( defined( $value ) ) {
+        $self->{childexit_} = $value;
+    }
+
+    return $self->{childexit_};
+}
+
 sub pipeready
 {
     my ( $self, $value ) = @_;
