@@ -10,9 +10,9 @@ use Fcntl;
 #
 # IMAP.pm --- a module to use POPFile for an IMAP connection.
 #
-# Copyright (c) 2001-2007 John Graham-Cumming
+# Copyright (c) 2001-2011 John Graham-Cumming
 #
-#   $Revision: 1.9.4.18 $
+#   $Revision: 3776 $
 #
 #   This file is part of POPFile
 #
@@ -121,7 +121,6 @@ sub initialize {
     $self->config_( 'update_interval', 20 );
     $self->config_( 'expunge', 0 );
     $self->config_( 'use_ssl', 0 );
-    $self->config_( 'use_multiple_connections', 0 );
 
     # Those next variables have getter/setter functions and should
     # not be used directly:
@@ -228,18 +227,6 @@ sub service {
 
     if ( time - $self->{last_update__} >= $self->config_( 'update_interval' ) ) {
 
-        # Should we use a single or multiple connections?
-        # And did this configuration options change?
-        if ( defined $self->{use_multiple_connections} ) {
-            my $config = $self->config_('use_multiple_connections');
-            if ( $config != $self->{use_multiple_connections} ) {
-                # Configuration has changed!
-                $self->disconnect_folders__();
-            }
-        }
-
-        $self->{use_multiple_connections} = $self->config_('use_multiple_connections');
-
         # Since the IMAP-Client module can throw an exception, i.e. die if
         # it detects a lost connection, we eval the following code to be able
         # to catch the exception. We also tell Perl to ignore broken pipes.
@@ -260,14 +247,7 @@ sub service {
                     $self->build_folder_list__();
                 }
 
-                # Try to establish connections, log in, and select for
-                # all of our folders if using multiple connections
-                if ( $self->{use_multiple_connections} ) {
-                    $self->connect_folders__();
-                }
-                else {
-                    $self->connect_server__();
-                }
+                $self->connect_server__();
 
                 # Reset the hash containing the hash values we have seen the
                 # last time through service.
@@ -288,7 +268,7 @@ sub service {
             $self->config_( 'training_mode', 0 );
 
             # say__() and get_response__() will die with this message:
-            if ( $@ =~ /^POPFILE-IMAP-EXCEPTION: (.+\)\))/ ) {
+            if ( $@ =~ /^POPFILE-IMAP-EXCEPTION: (.+\)\))/s ) {
                 $self->log_( 0, $1 );
             }
             # If we didn't die but somebody else did, we have empathy.
@@ -338,8 +318,8 @@ sub build_folder_list__ {
     %{$self->{folders__}} = ();
 
     # watched folders
-    foreach ( $self->watched_folders__() ) {
-        $self->{folders__}{$_}{watched} = 1;
+    foreach my $folder ( $self->watched_folders__() ) {
+        $self->{folders__}{$folder}{watched} = 1;
     }
 
     # output folders
@@ -366,105 +346,6 @@ sub build_folder_list__ {
 }
 
 
-
-#----------------------------------------------------------------------------
-# connect_folders__
-#
-#   This function will iterate over each folder found in the %{$self->{folders__}}
-#   hash. For each folder it will try to establish a connection, log in, and select
-#   the folder.
-#   The corresponding socket object, will be stored in
-#   $self->{folders__}{$folder}{imap}
-#
-# arguments:
-#   none.
-#
-# return value:
-#   none.
-#----------------------------------------------------------------------------
-
-sub connect_folders__ {
-    my $self = shift;
-
-    # Establish a connection for each folder in the hash
-    foreach my $folder ( keys %{$self->{folders__}} ) {
-
-        # We may already have a valid connection for this folder:
-        if ( exists $self->{folders__}{$folder}{imap} ) {
-            next;
-        }
-
-        $self->{folders__}{$folder}{server} = 1;
-        $self->{folders__}{$folder}{tag} = 0;
-
-        # The folder may be write-only:
-        if ( exists $self->{folders__}{$folder}{output}
-                &&
-            ! exists $self->{folders__}{$folder}{watched}
-                &&
-            $self->classifier()->is_pseudo_bucket( $self->api_session(),
-                                    $self->{folders__}{$folder}{output} ) ) {
-                next;
-        }
-
-        $self->log_( 1, "Trying to connect to ". $self->config_( 'hostname' ) . " for folder $folder." );
-        my $imap = $self->new_imap_client();
-
-        if ( $imap ) {
-            # Build a list of IMAP mailboxes if we haven't already got one:
-            unless ( @{$self->{mailboxes__}} ) {
-                @{$self->{mailboxes__}} = $imap->get_mailbox_list();
-            }
-            # Do a STATUS to check UIDVALIDITY and UIDNEXT
-            my $info = $imap->status( $folder );
-            my $uidnext = $info->{UIDNEXT};
-            my $uidvalidity = $info->{UIDVALIDITY};
-
-            if ( defined $uidvalidity && defined $uidnext ) {
-                # If we already have a UIDVALIDITY value stored,
-                # we compare the old and the new value.
-
-                if ( defined $imap->uid_validity( $folder ) ) {
-                    if ( $imap->check_uidvalidity( $folder, $uidvalidity ) ) {
-                        # That's the nice case. We simply do nothing.
-                    }
-                    else {
-                        # The validity has changed, we log this and update our stored
-                        # values for UIDNEXT and UIDVALIDITY
-                        $self->log_( 0, "Changed UIDVALIDITY for folder $folder. Some new messages might have been skipped." );
-                        $imap->uid_validity( $folder, $uidvalidity );
-                        $imap->uid_next( $folder, $uidnext );
-                    }
-                }
-                else {
-                    # We don't have a stored value, so let's change that.
-                    $self->log_( 0, "Storing UIDVALIDITY for folder $folder." );
-                    $imap->uid_validity( $folder, $uidvalidity );
-                    $imap->uid_next( $folder, $uidnext );
-                }
-            }
-            else {
-                $self->log_( 0, "Could not STATUS folder $folder." );
-                $imap->logout();
-                die "The connection to the IMAP server was lost";
-            }
-
-            # Now select the folder
-            if ( $imap->select( $folder ) ) {
-                $self->{folders__}{$folder}{imap} = $imap;
-            }
-            else {
-                $self->log_( 0, "Could not SELECT folder $folder." );
-                $imap->logout();
-            }
-        }
-        else {
-            # TODO: what now?
-        }
-    }
-}
-
-
 # ----------------------------------------------------------------------------
 #
 # connect_server__ - Connect to the IMAP server if we are only using a single
@@ -559,7 +440,7 @@ sub connect_server__ {
         else {
             $self->log_( 0, "Could not STATUS folder $folder." );
             $imap->logout();
-            die "POPFILE-IMAP-EXCEPTION: Lost connection while trying to log out (" . __FILE__ . '(' . __LINE__ . '))';
+            die "POPFILE-IMAP-EXCEPTION: Could not get a STATUS for IMAP folder $folder (" . __FILE__ . '(' . __LINE__ . '))';
         }
     }
 }
@@ -583,7 +464,10 @@ sub disconnect_folders__ {
 
         my $imap = $self->{folders__}{$folder}{imap};
         if ( defined $imap  && $imap->connected() ) {
-            $imap->logout( $folder );
+            # Workaround for POPFile crashes when disconnecting from server
+            eval {
+                $imap->logout( $folder );
+            };
         }
     }
     %{$self->{folders__}} = ();
@@ -615,7 +499,7 @@ sub scan_folder {
 
     # make the flags more accessible.
     my $is_watched = ( exists $self->{folders__}{$folder}{watched} ) ? 1 : 0;
-    my $is_output = ( exists $self->{folders__}{$folder}{output} ) ? $self->{folders__}{$folder}{output} : '';
+    my $is_output  = ( exists $self->{folders__}{$folder}{output } ) ? $self->{folders__}{$folder}{output} : '';
 
     $self->log_( 1, "Looking for new messages in folder $folder." );
 
@@ -630,12 +514,7 @@ sub scan_folder {
     my $moved_message = 0;
     my @uids = ();
 
-    if ( $self->{use_multiple_connections} ) {
-        @uids = $imap->get_new_message_list();
-    }
-    else {
-        @uids = $imap->get_new_message_list_unselected( $folder );
-    }
+    @uids = $imap->get_new_message_list_unselected( $folder );
 
     # We now have a list of messages with UIDs greater than or equal
     # to our last stored UIDNEXT value (of course, the list might be
@@ -1172,7 +1051,14 @@ sub can_reclassify__ {
 
                 # new and old bucket must be different
                 if ( $new_bucket ne $bucket ) {
-                    return $bucket;
+
+                    # The new bucket must not be a pseudo-bucket
+                    if ( ! $self->classifier()->is_pseudo_bucket( $self->api_session(), $new_bucket ) ) {
+                        return $bucket;
+                    }
+                    else {
+                        $self->log_( 1, "Will not reclassify to pseudo-bucket ($new_bucket)" );
+                    }
                 }
                 else {
                     $self->log_( 1, "Will not reclassify to same bucket ($new_bucket)." );
@@ -1429,12 +1315,7 @@ sub validate_item {
                     my $folder = $form->{ $key };
 
                     $bucket2folder{ $bucket } = $folder;
-
-                    # pseudo buckets are free to map wherever they like since
-                    # we will never reclassify to them anyway
-                    unless ( $self->classifier()->is_pseudo_bucket( $self->api_session(), $bucket ) ) {
-                        $folders{ $folder }++;
-                    }
+                    $folders{ $folder }++;
                 }
             }
 
@@ -1446,7 +1327,6 @@ sub validate_item {
                 }
                 else {
                     $self->folder_for_bucket__( $bucket, $folder );
-
                     $self->{folder_change_flag__} = 1;
                 }
             }
@@ -1476,7 +1356,7 @@ sub validate_item {
             # update interval
             my $form_interval = $form->{imap_options_update_interval};
             if ( defined $form_interval ) {
-                if ( $form_interval > 10 && $form_interval < 60*60 ) {
+                if ( $form_interval >= 10 && $form_interval <= 60*60 ) {
                     $self->config_( 'update_interval', $form_interval );
                     $templ->param( IMAP_if_interval_error => 0 );
                 }
