@@ -3,7 +3,7 @@
 #
 # bayes.pl --- Classify a mail message manually
 #
-# Copyright (c) 2001-2006 John Graham-Cumming
+# Copyright (c) 2001-2011 John Graham-Cumming
 #
 #   This file is part of POPFile
 #
@@ -60,11 +60,9 @@ if ( $#ARGV >= 0 ) {
         # Prevent the tool from finding another copy of POPFile running
 
         my $c = $POPFile->get_module('POPFile::Config');
+        my $current_piddir = $c->config_( 'piddir' );
         $c->config_( 'piddir', $c->config_( 'piddir' ) . 'bayes.pl.' );
 
-        # TODO: interface violation
-        $c->{save_needed__} = 0;
-
         $POPFile->CORE_start();
 
         my $b = $POPFile->get_module('Classifier::Bayes');
@@ -88,6 +86,12 @@ if ( $#ARGV >= 0 ) {
             }
         }
 
+        $c->config_( 'piddir', $current_piddir );
+
+        # Reload configuration file ( to avoid updating configurations )
+
+        $c->load_configuration();
+
         $b->release_session_key( $session );
         $POPFile->CORE_stop();
     }
